﻿// 2. Export
//
//	• Ein Sachbearbeiter ruft eine Konsolen-App auf, in welcher er die Kundennummer sowie den Zeitraum für die Abrechnung angibt
//	• Wird eine Aktion abgerechnet, darf diese zukünftig nicht nochmal berechnet werden (z.B. bei Überschneidung der Zeiträume)
//	• Die Ausgabe der Daten erfolgt als CSV-Datei. Der Dateiname folgt diesem Schema: Abrechnungsdatum_Kundennummer_vonDatum_bisDatum.csv
//	• Das Datum ist immer im folgenden Format anzugeben: yyyy-mm-dd
//	• Ein Abrechnungslauf muss folgende Daten umfassen:
//  	○ Zeitraum (von / bis Datum) der Abrechnung
//		○ Name und Nummer des Kunden
//		○ Gesamtsumme der Abrechnung
//		○ Übersicht der einzelnen Posten wie folgt:
//          Bereich	                    | Grundbetrag	| Anzahl	| Betrag
//          Newsletter wöchentlich	    | 0.001	        | 24	    | 0.024
//          Tarif Orange M 12 Monate	| 7.00	        | 0	        | 0.000
//          Tarif Orange S 12 Monate	| 5.00	        | 1	        | 5.000
//          Tarif Orange XL 12 Monate	| 12.00	        | 2	        | 24.000
//          Verkaufsgespräch	        | 0.030	        | 8	        | 0.240
//

using System;
using System.Collections.Generic;
using System.IO;
using MySql.Data.MySqlClient;

class Export
{
    static void Main(string[] args)
    {
        string customerNumber = args[0];
        string startDatum = args[1];
        string endDatum = args[2];

        ExportData(customerNumber, startDatum, endDatum);
    }

    static void ExportData(string kundennummer, string startDatum, string endDatum)
    {
        string myConnectionString = "server=localhost;uid=root;pwd=root;database=vr_contact";
        using (MySqlConnection myConnection = new MySqlConnection(myConnectionString))
        {
            try
            {
                myConnection.Open();

                // Kundendaten abrufen
                string clientQuery = "SELECT name, id FROM customers WHERE id = @clientId";
                MySqlCommand clientCmd = new MySqlCommand(clientQuery, myConnection);
                clientCmd.Parameters.AddWithValue("@clientId", kundennummer);

                using (MySqlDataReader clientReader = clientCmd.ExecuteReader())
                {
                    if (clientReader.Read())
                    {
                        string clientName = clientReader.GetString("name");
                        int clientId = clientReader.GetInt32("id");
                        clientReader.Close();

                        // Abrechnungsdaten abrufen
                        string invoiceQuery = @"
                            SELECT area, base_amount, quantity, amount 
                            FROM invoices 
                            WHERE client_id = @clientId 
                            AND invoice_date BETWEEN @startDatum AND @endDatum";

                        MySqlCommand invoiceCmd = new MySqlCommand(invoiceQuery, myConnection);
                        invoiceCmd.Parameters.AddWithValue("@clientId", clientId);
                        invoiceCmd.Parameters.AddWithValue("@startDatum", startDatum);
                        invoiceCmd.Parameters.AddWithValue("@endDatum", endDatum);

                        using (MySqlDataReader invoiceReader = invoiceCmd.ExecuteReader())
                        {
                            List<string> invoiceLines = new List<string>();
                            decimal totalAmount = 0;

                            while (invoiceReader.Read())
                            {
                                string area = invoiceReader.GetString("area");
                                decimal baseAmount = invoiceReader.GetDecimal("base_amount");
                                int quantity = invoiceReader.GetInt32("quantity");
                                decimal amount = invoiceReader.GetDecimal("amount");
                                totalAmount += amount;

                                invoiceLines.Add($"{area};{baseAmount};{quantity};{amount}");
                            }
                            invoiceReader.Close();

                            // CSV-Datei schreiben
                            string filename = $"{DateTime.Now:yyyy-MM-dd}_{kundennummer}_{startDatum}_{endDatum}.csv";
                            using (StreamWriter sw = new StreamWriter(filename))
                            {
                                sw.WriteLine($"Zeitraum: {startDatum} bis {endDatum}");
                                sw.WriteLine($"Name und Nummer des Kunden: {clientName}, {clientId}");
                                sw.WriteLine($"Gesamtsumme der Abrechnung: {totalAmount}");
                                sw.WriteLine("Bereich;Grundbetrag;Anzahl;Betrag");

                                foreach (string line in invoiceLines)
                                {
                                    sw.WriteLine(line);
                                }
                            }
                        }
                    }
                }
                myConnection.Close();
            }
            catch (MySqlException ex)
            {
                Console.WriteLine($"Database error: {ex.Message}");
            }
        }
    }
}

