﻿//Aufgabenstellung
//
//1.Import
//
//Als Teil des Entwicklungsteams sollen Sie einen Import für neue Nutzer der jeweiligen Kunden entwickeln.
//Die Kunden schicken zu diesem Zweck CSV-Dateien mit den Daten von neuen Nutzern.
//Diese Daten müssen in die bestehenden Datentabellen eingespielt werden.
//Dafür sind folgende Anforderungen formuliert:
//
//	• Ein Sachbearbeiter ruft eine Konsolen-App auf, in welcher er die Datei und die
//	    Kundennummer des Kunden als Parameter übergibt: Die Beispieldatei hat den Namen "import_yutani.csv" und
//	    gehört zum Kunden mit der Nummer "K2-002"
//
//	• Die Datei hat den Aufbau:
//      - Ein Nutzer mit Anrede, Vorname, Nachname und Geburtsdatum
//      - Ggf. Adresse: Straße mit Hausnummer, PLZ, Stadt
//      - Ggf. E-Mail
//      - Ggf. eine oder mehrere Telefonnummern: Vorwahl ohne führende 0 und Nummer
//
//    	• Ein Nutzer darf für einen Kunden nicht mehrmals importiert werden.
//
//    	• Die Daten sollen wie folgt validiert werden:
//    		○ Geburtsdatum: TT.MM.JJJJ
//    		○ Valide E-Mail-Adresse
//    		○ PLZ: exakt 5 Nummern
//    		○ Telefonnummer 3 bis 5 Nummern für die Vorwahl und 4 bis 10 Nummern für die Hauptnummer. Keine Sonderzeichen
//
//	• Nicht valide Datensätze von Nutzern sollen in einem Protokoll erfasst werden, ebenso bereits vorhandene Nutzer.
//	  Der komplette Datensatz eines Nutzers darf dann nicht importiert werden

/*
 * Das ist der Beispielcode für die Verbindung zum Datenbankserver
 * 
using MySql.Data;
using MySql.Data.MySqlClient;
using MySqlX.XDevAPI;
using System.Data.Common;
using System.Reflection.PortableExecutable;
using System.Xml;


    MySql.Data.MySqlClient.MySqlConnection myConnection;
    string myConnectionString;
    //set the correct values for your server, user, password and database name
    myConnectionString = "server=localhost;uid=root;pwd=root;database=vr_contact";

    try
    {
      myConnection = new MySql.Data.MySqlClient.MySqlConnection(myConnectionString);
      //open a connection
      myConnection.Open();

      // create a MySQL command and set the SQL statement with parameters
      MySqlCommand myCommand = new MySqlCommand();
    myCommand.Connection = myConnection;
      myCommand.CommandText = @"SELECT * FROM workshop WHERE maxParticipants = @code;";
      myCommand.Parameters.AddWithValue("@code", "12");

    // execute the command and read the results
    using MySqlDataReader myReader = myCommand.ExecuteReader();
      
        while (myReader.Read())
        {
          int id = myReader.GetInt32("maxParticipants");
        string name = myReader.GetString("title");
        Console.WriteLine(id + " " + name);
}
      
      myConnection.Close();
    }
    catch (MySql.Data.MySqlClient.MySqlException ex)
{
    
}
 * 
 */

using System;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using MySql.Data.MySqlClient;

class Import
{
    static void Main(string[] args)
    {
        string filePath = args[0];
        string customerNumber = args[1];

        ImportData(filePath, customerNumber);
    }

    static void ImportData(string filePath, string customerNumber)
    {
        List<string> errorLog = new List<string>();

        using (StreamReader sr = new StreamReader(filePath))
        {
            string line;
            while ((line = sr.ReadLine()) != null)
            {
                string[] parts = line.Split(';');
                User user = new User
                {
                    Gender = parts[0],
                    Firstname = parts[1],
                    Lastname = parts[2],
                    Birthdate = parts[3],
                    Address = new Address
                    {
                        Street = parts[4],
                        PostalCode = parts[5],
                        City = parts[6]
                    },
                    Email = parts[7],
                    Phones = new List<Phone>
                    {
                        new Phone { Prefix = parts[8], Number = parts[9] }
                    }
                };

                if (ValidateUser(user, out string error))
                {
                    InsertUser(user, customerNumber, errorLog);
                }
                else
                {
                    errorLog.Add(error);
                }
            }
        }

        File.WriteAllLines("error_log.txt", errorLog);
    }

    static bool ValidateUser(User user, out string error)
    {
        error = "";

        if (!Regex.IsMatch(user.Birthdate, @"^\d{2}\.\d{2}\.\d{4}$"))
        {
            error = "Invalid birthdate format.";
            return false;
        }

        if (!string.IsNullOrEmpty(user.Email) && !Regex.IsMatch(user.Email, @"^[^@\s]+@[^@\s]+\.[^@\s]+$"))
        {
            error = "Invalid email format.";
            return false;
        }

        if (!Regex.IsMatch(user.Address.PostalCode, @"^\d{5}$"))
        {
            error = "Invalid postal code format.";
            return false;
        }

        foreach (var phone in user.Phones)
        {
            if (!Regex.IsMatch(phone.Prefix, @"^\d{3,5}$") || !Regex.IsMatch(phone.Number, @"^\d{4,10}$"))
            {
                error = "Invalid phone number format.";
                return false;
            }
        }

        return true;
    }

    static void InsertUser(User user, string customerNumber, List<string> errorLog)
    {
        string myConnectionString = "server=localhost;uid=root;pwd=root;database=vr_contact";
        using (MySqlConnection myConnection = new MySqlConnection(myConnectionString))
        {
            try
            {
                myConnection.Open();

                // Check if user already exists for the customer
                string checkQuery = "SELECT COUNT(*) FROM users WHERE firstname = @firstname AND lastname = @lastname AND customer_id = @customer_id";
                MySqlCommand checkCmd = new MySqlCommand(checkQuery, myConnection);
                checkCmd.Parameters.AddWithValue("@firstname", user.Firstname);
                checkCmd.Parameters.AddWithValue("@lastname", user.Lastname);
                checkCmd.Parameters.AddWithValue("@customer_id", customerNumber);

                int userCount = Convert.ToInt32(checkCmd.ExecuteScalar());

                if (userCount > 0)
                {
                    errorLog.Add($"User {user.Firstname} {user.Lastname} already exists for customer {customerNumber}.");
                    return;
                }

                // Insert user
                string insertQuery = @"
                    INSERT INTO users (gender, firstname, lastname, birthdate, street, postal_code, city, email, customer_id) 
                    VALUES (@gender, @firstname, @lastname, @birthdate, @street, @postal_code, @city, @Email, @customer_id)";
                MySqlCommand insertCmd = new MySqlCommand(insertQuery, myConnection);
                insertCmd.Parameters.AddWithValue("@gender", user.Gender);
                insertCmd.Parameters.AddWithValue("@firstname", user.Firstname);
                insertCmd.Parameters.AddWithValue("@lastname", user.Lastname);
                insertCmd.Parameters.AddWithValue("@birthdate", user.Birthdate);
                insertCmd.Parameters.AddWithValue("@street", user.Address.Street);
                insertCmd.Parameters.AddWithValue("@postal_code", user.Address.PostalCode);
                insertCmd.Parameters.AddWithValue("@city", user.Address.City);
                insertCmd.Parameters.AddWithValue("@Email", user.Email);
                insertCmd.Parameters.AddWithValue("@customer_id", customerNumber);

                insertCmd.ExecuteNonQuery();
                myConnection.Close();
            }
            catch (MySqlException ex)
            {
                errorLog.Add($"Database error: {ex.Message}");
            }
        }
    }
}

class User
{
    public string Gender { get; set; }
    public string Firstname { get; set; }
    public string Lastname { get; set; }
    public string Birthdate { get; set; }
    public Address Address { get; set; }
    public string Email { get; set; }
    public List<Phone> Phones { get; set; } = new List<Phone>();
}

class Address
{
    public string Street { get; set; }
    public string PostalCode { get; set; }
    public string City { get; set; }
}

class Phone
{
    public string Prefix { get; set; }
    public string Number { get; set; }
}
